/**
 * fragment shader
 *
 * @author John Coffelt
 * @author Matthew Robertson
 *
 * modified for steps #3, #5, #8.0, #B1
 */
precision mediump float;

uniform sampler2D uTexture;

uniform float uAlpha;
uniform float uTime;

varying vec2 vTexcoords;


void main(void) {
    // initial version
    //    gl_FragColor = vec4(0.0, 0.0, 0.0, uAlpha);

    // version #3 receive the texture coordinates in the fragment shader
    //    gl_FragColor = vec4(vTexcoords.x, vTexcoords.y, 0.0, uAlpha);

    // version #5 sample from the texture in the shader
    //    gl_FragColor = texture2D(uTexture, vTexcoords);

    // version #8.0 set alpha to equal to uAlpha instead of texture's value
    //    vec4 texel = texture2D(uTexture, vTexcoords);
    //    texel.a = uAlpha;
    //    gl_FragColor = texel;

    // #B1 animate texture coordinates based on time
    vec2 coords = vec2(vTexcoords.x + uTime, vTexcoords.y + uTime);
    vec4 texel = texture2D(uTexture, coords);

    vec4 color = texel;
    color.a = uAlpha;

    gl_FragColor = color;

}
